//-----------　以下のコードは必要のあるユーザのみ記述する --------------

int FIELDCOLOR = #dddddd;

void drawCircle(viewportClass vp, float r, float width) {
  vp.ellipseMode(CENTER).noStroke().fill(#ffffff).ellipse(r + width / 2.0).fill(FIELDCOLOR).ellipse(r - width / 2.0);
  return;
}

void drawCourse(viewportClass vp) {
  float lw  = 19.0;      // [mm]
  float lw2 = lw / 2.0;

  // コース外枠
  vp.fill(FIELDCOLOR).stroke(#000000).rectMode(CORNERS).rect(-6000.0, -920.0, 6000.0, 920.0);
  // ライン
  vp.noStroke().fill(#ffffff   ).ellipseMode(RADIUS).arc(-4000.0, 0.0, 500.0 + lw2, 90.0, 270.0).arc(4000.0, 0.0, 500.0 + lw2, -90.0, 90.0).rect(-4000.0, 500.0 + lw2, 4000.0, -500.0 - lw2);
  vp.noStroke().fill(FIELDCOLOR).ellipseMode(RADIUS).arc(-4000.0, 0.0, 500.0 - lw2, 90.0, 270.0).arc(4000.0, 0.0, 500.0 - lw2, -90.0, 90.0).rect(-4000.0, 500.0 - lw2, 4000.0, -500.0 + lw2);
  // 円柱得点物
  vp.fill(#ffff00).stroke(#000000).ellipse(-5500.0, 0.0, 40.0).ellipse(5500.0, 0.0, 40.0);
  // ゴールエリア
  vp.moveTo(-2000.0, 0.0);
  drawCircle(vp, 500.0, lw);
  drawCircle(vp, 300.0, lw);
  drawCircle(vp, 100.0, lw);
  // DMC
  drawDMC(vp);
  // スタートゲート
  vp.fill(#ee2222).stroke(#000000).rectMode(RADIUS).rect(-3000.0, -500.0, 20.0, 240.0);
}

void drawDMC(viewportClass vp) {
  float a, da;
  float x, y, angle;
  x     = DMC.x;
  y     = DMC.y;
  angle = DMC.angle;
  vp.stroke(#000000).fill(#ffdd00, DMC.transparent ? 128 : 255).ellipseMode(RADIUS);
  vp.ellipse(x, y, DMC.DMC_RADIUS);
  da = 360.0 / 12.0;
  a  = da / 2.0 + angle;
  vp.fill(#ffff00);
  for (int i = 0; i < 12; i++) {
    a += da;
    vp.ellipse(x + DMC.DMC_RADIUS * cos(radians(a)), y + DMC.DMC_RADIUS * sin(radians(a)), DMC.MOTOR_RADIUS);
  }
  vp.fill(#ffffff);
  a = angle + 20.0;
  vp.ellipse(x + DMC.EYE_RADIUS * cos(radians(a)), y + DMC.EYE_RADIUS * sin(radians(a)), DMC.DMC_RADIUS * 0.15);
  a = angle - 20.0;
  vp.ellipse(x + DMC.EYE_RADIUS * cos(radians(a)), y + DMC.EYE_RADIUS * sin(radians(a)), DMC.DMC_RADIUS * 0.15);
}

void displayTextinfo() {
  int cx, cy, dy;
  cx = 50;
  cy = 25;
  dy = 30;
  crow.view("Text")._moveTo(cx, cy).textSize(24).viewTextColor(#ff0000)._text("Direct Motor Carの動作シミュレーション").viewTextColor(#000000).textSize(18);
  cy += dy * 2;
  crow.cv._write(cx, cy, "　中心座標： 　( " + nf(DMC.x / 1000.0, 0, 2) + ", " + nf(DMC.y / 1000.0, 0, 2) + " )");
  cy += dy;
  crow.cv._write(cx, cy, "　角度　　：　" + nf(DMC.angle, 0, 1) + "[deg]");
  cy += dy;
  crow.cv._write(cx, cy, "　速度　　：　" + nf(DMC.v, 0, 1) + "[mm/s]");
  cy += dy;
  crow.cv._write(cx, cy, "　時間　　：　" + nf(DMC.time, 0, 1) + "[s]");
  cy += dy * 2;
  crow.cv.textSize(14)._write(10, cy, "　キー操作： [4] 左回転　右回転 [6]　　[R] : 回転ON/OFF　[T] : 半透明ON/OFF　[Q] : 終了");
}

void Draw() {
  DMC.recalc();
  crow.clrViewAll(true);
  crow.view("DMC").worldCenter(DMC.x, DMC.y);
  if (DMC.rotate) crow.cv.viewRotate(-DMC.angle + 90.0); else crow.cv.viewRotate(0.0);
  drawCourse(crow.getView("Oval"));
  drawCourse(crow.getView("DMC"));
  // 軌跡の表示
  crow.view("Trajectory").lineTo(DMC.x, DMC.y);
  // テキスト情報の表示
  displayTextinfo();
}

// ユーザ独自のmousePressed()
void MousePressed() {
}
// ユーザ独自のmouseRelased()
void MouseReleased() {
}
// ユーザ独自のmouseDragged()
void MouseDragged() {
}
// ユーザ独自のmouseMoved()
void MouseMoved() {
}
// ユーザ独自のmouseClicked()
void MouseClicked() {
}
// ユーザ独自のkeyPressed()
void KeyPressed() {
  if (getKEY() == 'Q') crow.stop();
  if (key == '4') DMC.angle += 3.0;
  if (key == '6') DMC.angle -= 3.0;
  if (getKEY() == 'R') DMC.rotate = !DMC.rotate;
  if (getKEY() == 'T') DMC.transparent = !DMC.transparent;
}
// ユーザ独自のkeyRleased()
void KeyReleased() {
}
// ユーザ独自のkeyTyped()
void KeyTyped() {
}

// -----------

// もしMain()の開始前に実行しておきたいコードが別にある場合は以下に記述．
// このルーチンの出力はログファイルに記録されません．
void preMain() {
}

// もしMain()終了後に実行しておきたいコードが別にある場合は以下に記述．
// このルーチンの出力はログファイルに記録されません．
void postMain() {
}

